@@ -5,7 +5,6 @@ from datetime import timedelta
 from six import PY2
 from six.moves import zip_longest
 from procmon_parser.consts import Column, ColumnToOriginalName, RegistryOperation, NetworkOperation, ProcessOperation
-import tqdm
 
 
 SUPPORTED_COLUMNS = [
@@ -100,7 +99,7 @@ def are_we_better_than_procmon(pml_record, csv_record, column_name, pml_value, c
 def check_pml_equals_csv(csv_reader, pml_reader):
     first_event_date = None
     i = 0
-    for i, (csv_record, pml_record) in tqdm.tqdm(enumerate(zip_longest(csv_reader, pml_reader))):
+    for i, (csv_record, pml_record) in enumerate(zip_longest(csv_reader, pml_reader)):
         assert csv_record is not None, "PML reader has read more events then the CSV reader after {} records.".format(i)
         assert pml_record is not None, "CSV reader has read more events then the PML reader after {} records.".format(i)
 
